searchData={"items":[{"type":"module","title":"Aurora.Ctx.Core","doc":"Core implementation of common database operations using Ecto.\n\nProvides functions for:\n- CRUD operations (create, read, update, delete)\n- Pagination (list_paginated, navigation)\n- Query operations (count, list) with filtering, sorting, and preloading\n- Record management (change, new)\n- Query building and clause exclusion","ref":"Aurora.Ctx.Core.html"},{"type":"function","title":"Aurora.Ctx.Core.change/2","doc":"Creates a changeset for the given entity.\n\nParameters:\n- entity_or_changeset (Ecto.Schema.t() | Ecto.Changeset.t()) - Entity for changeset\n- attrs (map) - Changeset attributes\n\nReturns Ecto.Changeset.","ref":"Aurora.Ctx.Core.html#change/2"},{"type":"function","title":"Aurora.Ctx.Core.change/3","doc":"Creates a changeset for the given entity using a specific changeset function.","ref":"Aurora.Ctx.Core.html#change/3"},{"type":"function","title":"Parameters - Aurora.Ctx.Core.change/3","doc":"- entity_or_changeset (Ecto.Schema.t() | Ecto.Changeset.t()) - Existing record or changeset\n- changeset_function (atom()) - Custom changeset function\n- attrs (map()) - Changeset attributes","ref":"Aurora.Ctx.Core.html#change/3-parameters"},{"type":"function","title":"Returns - Aurora.Ctx.Core.change/3","doc":"- Ecto.Changeset.t()","ref":"Aurora.Ctx.Core.html#change/3-returns"},{"type":"function","title":"Examples - Aurora.Ctx.Core.change/3","doc":"alias MyApp.Product\n    product = %Product{name: \"Widget\"}\n    Core.change(product, %{name: \"New Widget\"})\n    #=> #Ecto.Changeset \n\n    Core.change(product, :custom_changeset, %{status: \"active\"})\n    #=> #Ecto.Changeset","ref":"Aurora.Ctx.Core.html#change/3-examples"},{"type":"function","title":"Aurora.Ctx.Core.count/3","doc":"Counts total records matching query conditions.\n\nParameters:\n- repo_module (module) - Ecto.Repo to use\n- schema_module (module) - Schema to query\n- opts (keyword) - Optional query filtering options\n\nReturns total count of matching records.","ref":"Aurora.Ctx.Core.html#count/3"},{"type":"function","title":"Aurora.Ctx.Core.create/3","doc":"Creates a new record.\n\nParameters:\n- repo_module (module()) - Ecto.Repo to use\n- schema_module (module() | ) - Schema to create\n- attrs (map()) - Attributes for the new record\n\nReturns {:ok, schema} on success, {:error, changeset} on failure.","ref":"Aurora.Ctx.Core.html#create/3"},{"type":"function","title":"Aurora.Ctx.Core.create/4","doc":"Creates a new record with optional custom changeset function.","ref":"Aurora.Ctx.Core.html#create/4"},{"type":"function","title":"Parameters - Aurora.Ctx.Core.create/4","doc":"- repo_module (module()) - Ecto.Repo module to use\n- schema_module (module() | Ecto.Changeset.t()) - Schema module for the record\n- changeset_function (atom()) - Custom changeset function to use\n- attrs (map()) - Attributes for the new record","ref":"Aurora.Ctx.Core.html#create/4-parameters"},{"type":"function","title":"Returns - Aurora.Ctx.Core.create/4","doc":"- `{:ok, Ecto.Schema.t()}` - On success\n- `{:error, Ecto.Changeset.t()}` - On validation failure","ref":"Aurora.Ctx.Core.html#create/4-returns"},{"type":"function","title":"Examples - Aurora.Ctx.Core.create/4","doc":"alias MyApp.{Repo, Product}\n    Core.create(Repo, Product, %{name: \"Widget\"})\n    #=> {:ok, %Product{name: \"Widget\"}}\n\n    Core.create(Repo, Product, :custom_changeset, %{\n      name: \"Widget\",\n      status: \"active\"\n    })\n    #=> {:ok, %Product{name: \"Widget\", status: \"active\"}}","ref":"Aurora.Ctx.Core.html#create/4-examples"},{"type":"function","title":"Aurora.Ctx.Core.create!/3","doc":"Creates a new record, raising on errors.\n\nParameters:\n- repo_module (module()) - Ecto.Repo to use\n- schema_module_or_changeset (module() | Ecto.Changeset.t()) - Schema to create\n- attrs (map()) - Attributes for the new record\n\nReturns created schema or raises on error.","ref":"Aurora.Ctx.Core.html#create!/3"},{"type":"function","title":"Aurora.Ctx.Core.create!/4","doc":"Creates a new record, raising on errors.\n\nParameters:\n- repo_module (module) - Ecto.Repo to use\n- schema_module_or_changeset (module | Ecto.Changeset) - Schema to create\n- changeset_function (atom) - Changeset function to use\n- attrs (map) - Attributes for the new record\n\nReturns created schema or raises on error.","ref":"Aurora.Ctx.Core.html#create!/4"},{"type":"function","title":"Aurora.Ctx.Core.delete/2","doc":"Deletes the given entity.\n\nParameters:\n- repo_module (module): The Ecto.Repo module to use\n- entity (Ecto.Schema.t()): The entity to delete\n\nReturns: {:ok, Ecto.Schema.t()} | {:error, Ecto.Changeset.t()}","ref":"Aurora.Ctx.Core.html#delete/2"},{"type":"function","title":"Aurora.Ctx.Core.delete!/2","doc":"Deletes the given entity, raising on errors.\n\nParameters:\n- repo_module (module): The Ecto.Repo module to use\n- entity (Ecto.Schema.t()): The entity to delete\n\nReturns: Ecto.Schema.t()\nRaises: Ecto.StaleEntryError","ref":"Aurora.Ctx.Core.html#delete!/2"},{"type":"function","title":"Aurora.Ctx.Core.exclude_clauses/2","doc":"Excludes query clauses from an Ecto query.\n\nParameters:\n- query (Ecto.Query.t()) - Query to modify\n- clauses (atom | [atom]) - Clause(s) to exclude. Available clauses:\n  - :where - WHERE conditions\n  - :select - SELECT clauses\n  - :order_by - ORDER BY clauses\n  - :group_by - GROUP BY clauses\n  - :having - HAVING conditions\n  - :limit - LIMIT clause\n  - :offset - OFFSET clause\n  - :preload - Preload associations\n  - :lock - Lock clauses\n\nReturns modified query.","ref":"Aurora.Ctx.Core.html#exclude_clauses/2"},{"type":"function","title":"Examples - Aurora.Ctx.Core.exclude_clauses/2","doc":"query = from(p in Product)\n    exclude_clauses(query, :where)\n    exclude_clauses(query, [:select, :order_by])","ref":"Aurora.Ctx.Core.html#exclude_clauses/2-examples"},{"type":"function","title":"Aurora.Ctx.Core.get/4","doc":"Gets a record by id.\n\nParameters:\n- repo_module (module) - Ecto.Repo to use\n- schema_module (module) - Schema to query\n- id (term) - Primary key to look up\n- opts (keyword) - Optional query parameters\n\nReturns found schema or nil if not found.","ref":"Aurora.Ctx.Core.html#get/4"},{"type":"function","title":"Aurora.Ctx.Core.get!/4","doc":"Gets a record by id, raising if not found.\n\nParameters:\n- repo_module (module) - Ecto.Repo to use\n- schema_module (module) - Schema to query\n- id (term) - Primary key to look up\n- opts (keyword) - Optional query parameters\n\nReturns found schema or raises Ecto.NoResultsError.","ref":"Aurora.Ctx.Core.html#get!/4"},{"type":"function","title":"Aurora.Ctx.Core.list/3","doc":"Lists all records for a schema with optional filtering and sorting.","ref":"Aurora.Ctx.Core.html#list/3"},{"type":"function","title":"Parameters - Aurora.Ctx.Core.list/3","doc":"- repo_module (module()) - Ecto.Repo module to use\n- schema_module (module()) - Schema module to query\n- opts (keyword()) - Optional query parameters (see `Aurora.Ctx.QueryBuilder` for available options)","ref":"Aurora.Ctx.Core.html#list/3-parameters"},{"type":"function","title":"Returns - Aurora.Ctx.Core.list/3","doc":"- list(Ecto.Schema.t())","ref":"Aurora.Ctx.Core.html#list/3-returns"},{"type":"function","title":"Examples - Aurora.Ctx.Core.list/3","doc":"alias MyApp.{Repo, Product}\n    Core.list(Repo, Product)\n    #=> [%Product{}, ...]\n\n    Core.list(Repo, Product,\n      where: [status: :active],\n      order_by: [desc: :inserted_at],\n      preload: [:category]\n    )\n    #=> [%Product{category: %Category{}}, ...]","ref":"Aurora.Ctx.Core.html#list/3-examples"},{"type":"function","title":"Aurora.Ctx.Core.list_paginated/3","doc":"Lists records with pagination support.","ref":"Aurora.Ctx.Core.html#list_paginated/3"},{"type":"function","title":"Parameters - Aurora.Ctx.Core.list_paginated/3","doc":"- repo_module (module()) - Ecto.Repo module to use\n- schema_module (module()) - Schema module to query\n- opts (keyword()) - See `Aurora.Ctx.QueryBuilder` for available query options.","ref":"Aurora.Ctx.Core.html#list_paginated/3-parameters"},{"type":"function","title":"Configuration - Aurora.Ctx.Core.list_paginated/3","doc":"Default pagination settings can be configured in your `config.exs`:\n\n    config :aurora_ctx, :pagination,\n      page: 1,\n      per_page: 40\n\nIf not configured, defaults to page: 1, per_page: 40.\nPage navigation is safe - attempting to navigate beyond valid pages will return\nthe current page unchanged.","ref":"Aurora.Ctx.Core.html#list_paginated/3-configuration"},{"type":"function","title":"Returns - Aurora.Ctx.Core.list_paginated/3","doc":"- `Aurora.Ctx.Pagination` struct.","ref":"Aurora.Ctx.Core.html#list_paginated/3-returns"},{"type":"function","title":"Examples - Aurora.Ctx.Core.list_paginated/3","doc":"alias MyApp.{Repo, Product}\n    Core.list_paginated(Repo, Product,\n      paginate: %{page: 1, per_page: 20},\n      where: [category_id: 1],\n      order_by: [desc: :inserted_at]\n    )\n    #=> %Aurora.Ctx.Pagination{\n    #=>   entries: [%Product{}],\n    #=>   page: 1,\n    #=>   per_page: 20,\n    #=>   entries_count: 50,\n    #=>   pages_count: 3\n    #=> }","ref":"Aurora.Ctx.Core.html#list_paginated/3-examples"},{"type":"function","title":"Aurora.Ctx.Core.new/3","doc":"Initializes a new schema struct with optional attributes or options.","ref":"Aurora.Ctx.Core.html#new/3"},{"type":"function","title":"Parameters - Aurora.Ctx.Core.new/3","doc":"- repo_module (module) - Ecto.Repo to use\n- schema_module (module) - Schema to initialize\n- attrs_or_opts - One of:\n  - map: Initial attributes (will use empty options)\n  - keyword list: Options (will use empty attributes)\n\nReturns initialized schema struct.","ref":"Aurora.Ctx.Core.html#new/3-parameters"},{"type":"function","title":"Aurora.Ctx.Core.new/4","doc":"Initializes a new struct with optional preloads.\n\nParameters:\n- repo_module (module): The Ecto.Repo module to use\n- schema_module (module): The Ecto.Schema module to initialize\n- opts (keyword): Optional preload parameters\n\nReturns: Ecto.Schema.t()","ref":"Aurora.Ctx.Core.html#new/4"},{"type":"function","title":"Aurora.Ctx.Core.next_page/1","doc":"Moves to next page in paginated results.\n\nParameters:\n  - paginate (Aurora.Ctx.Pagination.t()) - Current pagination state containing\n    entries, page number, and other metadata. If already on the last page,\n    returns the current page unchanged.\n\nExample:\n    paginate = list_products_paginated(per_page: 20)\n    next_page = next_page(paginate) # Moves to page 2","ref":"Aurora.Ctx.Core.html#next_page/1"},{"type":"function","title":"Aurora.Ctx.Core.previous_page/1","doc":"Moves to previous page in paginated results.\n\nParameters:\n  - paginate (Aurora.Ctx.Pagination.t()) - Current pagination state containing\n    entries, page number, and other metadata. If already on page 1,\n    returns the first page unchanged.\n\nExample:\n    paginate = list_products_paginated(page: 2, per_page: 20)\n    prev_page = previous_page(paginate) # Moves back to page 1","ref":"Aurora.Ctx.Core.html#previous_page/1"},{"type":"function","title":"Aurora.Ctx.Core.to_page/2","doc":"Changes to a specific page in paginated results.\n\nParameters:\n  - paginate (Aurora.Ctx.Pagination.t()) - Current pagination state\n  - page (integer) - Target page number. If the requested page is out of range\n    (< 1 or > pages_count), returns the current page unchanged.\n\nExample:\n    paginate = list_products_paginated(per_page: 20)\n    page5 = to_page(paginate, 5)  # Jumps directly to page 5\n\n    # Out of range page results in no change\n    page5 = to_page(paginate, 999) # Returns current page if 999 > pages_count\n\nReturns:\n  - Aurora.Ctx.Pagination struct with entries for the target page, or unchanged\n    if the target page is out of range.","ref":"Aurora.Ctx.Core.html#to_page/2"},{"type":"function","title":"Aurora.Ctx.Core.update/3","doc":"Updates an entity with given attributes.\n\nParameters:\n- repo_module (module) - Ecto.Repo to use\n- entity_or_changeset (Ecto.Schema.t() | Ecto.Changeset.t()) - Entity to update or changeset to apply\n- attrs (map) - Update attributes\n\nReturns {:ok, schema} on success, {:error, changeset} on failure.","ref":"Aurora.Ctx.Core.html#update/3"},{"type":"function","title":"Aurora.Ctx.Core.update/4","doc":"Updates an entity using a specific changeset function.","ref":"Aurora.Ctx.Core.html#update/4"},{"type":"function","title":"Parameters - Aurora.Ctx.Core.update/4","doc":"- repo_module (module()) - Ecto.Repo module to use\n- entity_or_changeset (Ecto.Schema.t() | Ecto.Changeset.t()) - Existing record to update\n- changeset_function (atom()) - Custom changeset function\n- attrs (map()) - Update attributes","ref":"Aurora.Ctx.Core.html#update/4-parameters"},{"type":"function","title":"Returns - Aurora.Ctx.Core.update/4","doc":"- `{:ok, Ecto.Schema.t()}` - On success\n- `{:error, Ecto.Changeset.t()}` - On validation failure","ref":"Aurora.Ctx.Core.html#update/4-returns"},{"type":"function","title":"Examples - Aurora.Ctx.Core.update/4","doc":"alias MyApp.{Repo, Product}\n    product = %Product{name: \"Old Name\"}\n    Core.update(Repo, product, %{name: \"New Name\"})\n    #=> {:ok, %Product{name: \"New Name\"}}","ref":"Aurora.Ctx.Core.html#update/4-examples"},{"type":"module","title":"Aurora.Ctx.Pagination","doc":"Struct for handling pagination state.","ref":"Aurora.Ctx.Pagination.html"},{"type":"module","title":"Fields - Aurora.Ctx.Pagination","doc":"* `repo_module` - Repository module used for database operations\n* `schema_module` - Schema module representing the data being paginated\n* `entries_count` - Total number of records available\n* `pages_count` - Total number of pages based on entries_count and per_page\n* `page` - Current page number (default: 1)\n* `per_page` - Number of entries per page (default: 40)\n* `opts` - Additional query options passed to database operations\n* `entries` - List of records for the current page","ref":"Aurora.Ctx.Pagination.html#module-fields"},{"type":"module","title":"Configuration - Aurora.Ctx.Pagination","doc":"Default pagination settings can be configured in your `config.exs`:\n\n    config :aurora_ctx, :pagination,\n      page: 1,\n      per_page: 40\n\nIf not configured, defaults to page: 1, per_page: 40.","ref":"Aurora.Ctx.Pagination.html#module-configuration"},{"type":"function","title":"Aurora.Ctx.Pagination.new/0","doc":"Creates a new pagination struct with safe defaults from the application configuration.\nDelegates to new/1 with the configured defaults.\n\nAny invalid configuration values will be replaced with system defaults:\n- page: 1\n- per_page: 40\n\nReturns:\n  t()","ref":"Aurora.Ctx.Pagination.html#new/0"},{"type":"function","title":"Aurora.Ctx.Pagination.new/1","doc":"Creates a new pagination struct with the given attributes.\n\nParameters:\n  - attrs (map | keyword): Pagination attributes\n    - :page (pos_integer): Page number (default: 1)\n    - :per_page (pos_integer): Items per page (default: 40)\n    - :opts (keyword): Additional query options to be passed to database operations\n    - :entries (list): Initial list of entries for the current page\n\nAny invalid or negative values for :page or :per_page will be replaced with defaults.\nEmpty or invalid :opts and :entries will be initialized as empty lists.\n\nReturns:\n  t()","ref":"Aurora.Ctx.Pagination.html#new/1"},{"type":"type","title":"Aurora.Ctx.Pagination.t/0","doc":"","ref":"Aurora.Ctx.Pagination.html#t:t/0"},{"type":"task","title":"mix ctx.test.setup","doc":"A Mix task for setting up the test environment by creating and migrating the test database.","ref":"Mix.Tasks.Ctx.Test.Setup.html"},{"type":"function","title":"Mix.Tasks.Ctx.Test.Setup.run/1","doc":"Runs the test setup task by creating and migrating the test database.\n\nParameters:\n- args: list | nil - Command line arguments (not used)\n\nReturns:\n- any","ref":"Mix.Tasks.Ctx.Test.Setup.html#run/1"},{"type":"module","title":"Aurora.Ctx","doc":"A module for automatically generating CRUD operations for Ecto schemas.\n\nThis module provides macros to generate standard database operation functions\nbased on Ecto schemas. It simplifies the creation of context modules by\nautomatically implementing common CRUD operations.","ref":"Aurora.Ctx.html"},{"type":"module","title":"Usage - Aurora.Ctx","doc":"defmodule MyApp.Accounts do\n      use Aurora.Ctx\n\n      ctx_register_schema(User)\n      # or with options\n      ctx_register_schema(User, CustomRepo,\n        update_changeset: :custom_changeset,\n        create_changeset: :create_changeset\n      )\n    end\n\nFor detailed examples of using the generated functions, see [Examples Guide](examples.html).","ref":"Aurora.Ctx.html#module-usage"},{"type":"module","title":"Generated Functions - Aurora.Ctx","doc":"For a schema with source name \"users\" and module name \"user\", the following functions are generated:","ref":"Aurora.Ctx.html#module-generated-functions"},{"type":"module","title":"List Functions - Aurora.Ctx","doc":"- `list_users/0` - List all records\n- `list_users/1` - List all records with options\n- `list_users_paginated/0` - List records with pagination\n- `list_users_paginated/1` - List records with pagination and options\n- `count_users/0` - Count total records\n- `count_users/1` - Count records with options","ref":"Aurora.Ctx.html#module-list-functions"},{"type":"module","title":"Pagination Functions - Aurora.Ctx","doc":"- `to_users_page/2` - Navigate to a specific page\n- `next_users_page/1` - Navigate to the next page\n- `previous_users_page/1` - Navigate to the previous page","ref":"Aurora.Ctx.html#module-pagination-functions"},{"type":"module","title":"Create Functions - Aurora.Ctx","doc":"- `create_user/0` - Create a record with empty attributes\n- `create_user/1` - Create a record with given attributes\n- `create_user!/0` - Create a record with empty attributes (raises on error)\n- `create_user!/1` - Create a record with given attributes (raises on error)","ref":"Aurora.Ctx.html#module-create-functions"},{"type":"module","title":"Get Functions - Aurora.Ctx","doc":"- `get_user/1` - Get a record by ID\n- `get_user/2` - Get a record by ID with options\n- `get_user!/1` - Get a record by ID (raises if not found)\n- `get_user!/2` - Get a record by ID with options (raises if not found)","ref":"Aurora.Ctx.html#module-get-functions"},{"type":"module","title":"Delete Functions - Aurora.Ctx","doc":"- `delete_user/1` - Delete a record\n- `delete_user!/1` - Delete a record (raises on error)","ref":"Aurora.Ctx.html#module-delete-functions"},{"type":"module","title":"Change Functions - Aurora.Ctx","doc":"- `change_user/1` - Create a changeset from a record\n- `change_user/2` - Create a changeset from a record with attributes","ref":"Aurora.Ctx.html#module-change-functions"},{"type":"module","title":"Update Functions - Aurora.Ctx","doc":"- `update_user/1` - Update a record\n- `update_user/2` - Update a record with attributes","ref":"Aurora.Ctx.html#module-update-functions"},{"type":"module","title":"New Functions - Aurora.Ctx","doc":"- `new_user/0` - Initialize a new struct\n- `new_user/1` - Initialize a new struct with attributes\n- `new_user/2` - Initialize a new struct with attributes and options","ref":"Aurora.Ctx.html#module-new-functions"},{"type":"macro","title":"Aurora.Ctx.ctx_register_schema/2","doc":"Registers a schema for CRUD function generation.","ref":"Aurora.Ctx.html#ctx_register_schema/2"},{"type":"macro","title":"Parameters - Aurora.Ctx.ctx_register_schema/2","doc":"* `schema_module` (module()) - The Ecto schema module to generate functions for\n  * `repo` (module() | nil) - (Optional) The Ecto.Repo to use for database operations. Can be set globally with `@ctx_repo_module`\n  * `opts` (keyword()) - (Optional) Configuration options:\n    * `:changeset` (atom()) - A specific function to use for change function changesets (default: `:changeset`)\n    * `:create_changeset` (atom()) - A specific function to use for creation changesets (defaults to the value of option `:changeset` or the function `:changeset`)\n    * `:infix` (binary) - The fixed part to use when constructing the functions' names.\n        By default, is the lowercase of the module name (the last part of the full module name).\n    * `:plural_infix` (binary) - The fixed part to use when constructing the functions' names for plural functions.\n        By default, uses the table name defined in the schema module.\n    * `:update_changeset` (atom()) - The function to use for changesets (defaults to the value of option `:changeset` or the function `:changeset`)","ref":"Aurora.Ctx.html#ctx_register_schema/2-parameters"},{"type":"macro","title":"Repository Configuration - Aurora.Ctx.ctx_register_schema/2","doc":"The Ecto.Repo can be configured in two ways:\n  1. Passing it as the second argument to `ctx_register_schema/3`\n  2. Setting the `@ctx_repo_module` module attribute in your context\n\nIf neither is specified, it will attempt to use `YourApp.Repo` based on your\ncontext module's namespace.","ref":"Aurora.Ctx.html#ctx_register_schema/2-repository-configuration"},{"type":"macro","title":"Function Names - Aurora.Ctx.ctx_register_schema/2","doc":"Given a User schema with table \"users\":\n\n    # Default naming (no options)\n    get_user/1          # Singular functions use module name\n    list_users/0        # Plural functions use table name\n\n    # With infix: \"customer\"\n    get_customer/1      # Singular functions use custom infix\n    list_users/0        # Plural functions still use table name\n\n    # With plural_infix: \"customers\"\n    get_user/1          # Singular functions use module name\n    list_customers/0    # Plural functions use custom plural infix\n\n    # With both infix: \"customer\" and plural_infix: \"customers\"\n    get_customer/1      # Singular functions use custom infix\n    list_customers/0    # Plural functions use custom plural infix","ref":"Aurora.Ctx.html#ctx_register_schema/2-function-names"},{"type":"macro","title":"Examples - Aurora.Ctx.ctx_register_schema/2","doc":"# Using explicit repo\n    ctx_register_schema(User, MyApp.Repo)\n\n    # Using module attribute\n    @ctx_repo_module MyApp.Repo\n    ctx_register_schema(User)\n\n    # Generating custom function names\n    ctx_register_schema(User, MyApp.Repo,\n      infix: \"customer\",\n      plural_infix: \"customers\"\n    )","ref":"Aurora.Ctx.html#ctx_register_schema/2-examples"},{"type":"macro","title":"Aurora.Ctx.ctx_register_schema/3","doc":"","ref":"Aurora.Ctx.html#ctx_register_schema/3"},{"type":"function","title":"Aurora.Ctx.implementable_functions/2","doc":"Lists all implementable CRUD functions for a given schema.\n\nThis function is used internally to determine which functions should be generated\nfor a given schema. It returns a list of maps containing function definitions.","ref":"Aurora.Ctx.html#implementable_functions/2"},{"type":"function","title":"Parameters - Aurora.Ctx.implementable_functions/2","doc":"* `schema_module` - The Ecto schema module to analyze\n  * `opts` - Options that affect function generation:\n    * `:infix` - Custom infix for singular function names\n    * `:plural_infix` - Custom infix for plural function names","ref":"Aurora.Ctx.html#implementable_functions/2-parameters"},{"type":"function","title":"Returns - Aurora.Ctx.implementable_functions/2","doc":"* List of maps containing:\n    * `:type` - The function type:\n      * `:list` - Functions that return collections of records\n      * `:list_paginated` - Functions that return paginated results\n      * `:count` - Functions that count records\n      * `:create` - Functions that create new records\n      * `:get` - Functions that fetch single records\n      * `:delete` - Functions that remove records\n      * `:change` - Functions that create changesets\n      * `:update` - Functions that modify records\n      * `:new` - Functions that initialize structs\n      * `:to_page`, `:next_page`, `:previous_page` - Pagination navigation\n    * `:name` - The generated function name as an atom\n    * `:arity` - The function arity (number of arguments)","ref":"Aurora.Ctx.html#implementable_functions/2-returns"},{"type":"module","title":"Aurora.Ctx.QueryBuilder","doc":"Provides functions for building and composing Ecto queries with common operations like filtering,\nsorting, pagination, and preloading associations.","ref":"Aurora.Ctx.QueryBuilder.html"},{"type":"function","title":"Aurora.Ctx.QueryBuilder.options/2","doc":"Applies a list of query options to an Ecto query.","ref":"Aurora.Ctx.QueryBuilder.html#options/2"},{"type":"function","title":"Parameters - Aurora.Ctx.QueryBuilder.options/2","doc":"- query (Ecto.Query.t() | nil) - Core query to modify\n- options (keyword) - List of query options to apply","ref":"Aurora.Ctx.QueryBuilder.html#options/2-parameters"},{"type":"function","title":"Options - Aurora.Ctx.QueryBuilder.options/2","doc":"- :preload (atom | list) - Associations to preload\n- :where (keyword | map | tuple) - Filter conditions\n  - Simple equality: `{:field, value}`\n  - Comparison: `{:field, operator, value}` where operator can be:\n    - :greater_than, :gt - Greater than\n    - :greater_equal_than, :ge - Greater than or equal\n    - :less_than, :lt - Less than\n    - :less_equal_than, :le - Less than or equal\n    - :equal_to, :eq - Equal to\n  - Range: `{:field, :between, start_value, end_value}`\n- :or_where (keyword | map | tuple) - Same as :where but combines with OR\n- :paginate (map) - Pagination options with keys:\n  - :page (integer) - Page number\n  - :per_page (integer) - Items per page\n- :sort (atom | tuple | list) - Sorting options:\n  - field (atom | {:asc | :desc, field}) - Field to sort by ascending\n  - fields ([{:asc | :desc, field}]) - List of sort fields specifications","ref":"Aurora.Ctx.QueryBuilder.html#options/2-options"},{"type":"function","title":"Returns - Aurora.Ctx.QueryBuilder.options/2","doc":"- Ecto.Query.t() - Modified query with all options applied","ref":"Aurora.Ctx.QueryBuilder.html#options/2-returns"},{"type":"function","title":"Examples - Aurora.Ctx.QueryBuilder.options/2","doc":"query = from(p in Product)\n    QueryBuilder.options(query,\n      where: [status: :active],\n      preload: [:category],\n      sort: [desc: :inserted_at],\n      paginate: %{page: 1, per_page: 20}\n    )\n\nFor more advanced query examples including filtering, sorting, and associations,\nsee the [Query Usage Guide](examples.html#query-usage).","ref":"Aurora.Ctx.QueryBuilder.html#options/2-examples"},{"type":"extras","title":"Changelog for v0.x","doc":"# Changelog for v0.x","ref":"changelog.html"},{"type":"extras","title":"v0.1.3 - Changelog for v0.x","doc":"Requires Elixir v1.17+\nCompatible with Ecto 3.12+","ref":"changelog.html#v0-1-3"},{"type":"extras","title":"Documentation update and fixes - Changelog for v0.x","doc":"- README.md - logo added.\n- Hex docs - logo added.\n- Functions.md - improved to reflect new functionalities.\n- Examples.md - Phoenix Liveview example added.\n- Modules - Modules documentation updated.\n- Identity assets - logo and icons added.","ref":"changelog.html#documentation-update-and-fixes"},{"type":"extras","title":"Dependencies upgrade - Changelog for v0.x","doc":"- ex_doc upgraded and locked from 0.38.1 to 0.38.2","ref":"changelog.html#dependencies-upgrade"},{"type":"extras","title":"v0.1.2 - Changelog for v0.x","doc":"Requires Elixir v1.17+\nCompatible with Ecto 3.12+","ref":"changelog.html#v0-1-2"},{"type":"extras","title":"Documentation update and fixes - Changelog for v0.x","doc":"- README.md - more details added.\n- overview.md - description of pagination functions added.\n- functions.md - description of pagination functions.\n- examples.md - Added Phoenix examples of pagination functions, improved multiple changeset examples.","ref":"changelog.html#documentation-update-and-fixes-1"},{"type":"extras","title":"Features - Changelog for v0.x","doc":"- Added a `:changeset` option to ctx_register_schema. \n  This provides a default changeset for create, update and change functions.","ref":"changelog.html#features"},{"type":"extras","title":"v0.1.1 - Changelog for v0.x","doc":"Requires Elixir v1.17+\nCompatible with Ecto 3.2+","ref":"changelog.html#v0-1-1"},{"type":"extras","title":"Fixes - Changelog for v0.x","doc":"- Mix task test.setup renamed to ctx.test.setup, to avoid conflicts with other apps or libraries.","ref":"changelog.html#fixes"},{"type":"extras","title":"Features - Changelog for v0.x","doc":"- Two new options added to ctx_register_schema\n    - :infix and :plural_infix to change the names that are generated.\n- Implemented create_changeset and update_changeset.\n- Navigation functions added:\n    - to_(plural_inflix)_page/2, next_(plural_inflix)_page/1, previous_(plural_inflix)_page/1","ref":"changelog.html#features-1"},{"type":"extras","title":"Upgraded dependencies - Changelog for v0.x","doc":"- ex_doc - 0.37.2 to 0.38.3","ref":"changelog.html#upgraded-dependencies"},{"type":"extras","title":"v0.1.0 - Changelog for v0.x","doc":"Requires Elixir v1.17+\nCompatible with Ecto 3.2+","ref":"changelog.html#v0-1-0"},{"type":"extras","title":"Features - Changelog for v0.x","doc":"- Automatic CRUD function generation for Ecto schemas\n- Customizable repository module selection\n- Customizable changeset function names\n- Support for both safe and raising versions of operations\n- Built-in support for preloading associations","ref":"changelog.html#features-2"},{"type":"extras","title":"Overview","doc":"# Overview\n\nAurora.Ctx is a code generation library that implements the Context pattern in Phoenix applications. It automatically generates standardized database interaction functions following Phoenix's architectural guidelines, while maintaining full compatibility with Ecto's query composition and changesets. The library supports both standard CRUD operations and advanced query features like filtering, sorting, and pagination.","ref":"overview.html"},{"type":"extras","title":"Key Benefits - Overview","doc":"- **Reduced Boilerplate**: Automatically generates common database access functions\n- **Consistent API**: Ensures uniform function names and patterns across contexts\n- **Type Safety**: All generated functions include proper typespecs\n- **Flexible**: Supports customization through options and hooks\n- **Maintainable**: Less code to maintain, centralized logic","ref":"overview.html#key-benefits"},{"type":"extras","title":"Getting Started - Overview","doc":"Add `aurora_ctx` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:aurora_ctx, \"~> 0.1.3\"}\n  ]\nend\n```","ref":"overview.html#getting-started"},{"type":"extras","title":"Basic Usage - Overview","doc":"","ref":"overview.html#basic-usage"},{"type":"extras","title":"Creating an Ecto Schema - Overview","doc":"> **Tip**: Consider using [TypedEctoSchema](https://hexdocs.pm/typed_ecto_schema) to reduce boilerplate in your schema definitions and add compile-time type checking.\n\nLet's start with a typical Ecto schema definition that we'll use throughout this overview:\n\n```elixir\ndefmodule MyApp.Inventory.Product do\n  use Ecto.Schema\n  \n  schema \"products\" do\n    field :name, :string\n    field :price, :decimal\n    \n    timestamps()\n  end\n\n  def changeset(product, attrs) do\n    product\n    |> cast(attrs, [:name, :price])\n    |> validate_required([:name, :price])\n  end\nend\n```","ref":"overview.html#creating-an-ecto-schema"},{"type":"extras","title":"Traditional Context Approach - Overview","doc":"Without Aurora.Ctx, you would typically write a context module with all the CRUD operations manually:\n\n```elixir\ndefmodule MyApp.Inventory do\n  import Ecto.Query\n  alias MyApp.Repo\n  alias MyApp.Inventory.Product\n\n  def list_products do\n    Product |> from() |> Repo.all()\n  end\n\n  def get_product(id) do\n    Product |> Repo.get(id)\n  end\n\n  def get_product!(id) do\n    Product |> Repo.get!(id)\n  end\n\n  def create_product(attrs) do\n    %Product{}\n    |> Product.changeset(attrs)\n    |> Repo.insert()\n  end\n\n  def update_product(%Product{} = product, attrs) do\n    product\n    |> Product.changeset(attrs)\n    |> Repo.update()\n  end\n\n  def delete_product(%Product{} = product) do\n    Repo.delete(product)\n  end\n\n  def change_product(%Product{} = product, attrs \\\\ %{}) do\n    Product.changeset(product, attrs)\n  end\nend\n```","ref":"overview.html#traditional-context-approach"},{"type":"extras","title":"Context Generation with Aurora.Ctx - Overview","doc":"With Aurora.Ctx, the same functionality can be achieved with just a few lines of code:\n\n```elixir\ndefmodule MyApp.Inventory do\n  use Aurora.Ctx\n\n  # The repository will default to MyApp.Repo  \n  ctx_register_schema(MyApp.Inventory.Product)\nend\n```\n\nNow you can use the context just like the traditional approach:\n\n```elixir\n# List all products\nproducts = Inventory.list_products()\n\n# Get a specific product\nproduct = Inventory.get_product!(1)\n\n# Create a new product\n{:ok, product} = Inventory.create_product(%{name: \"Widget\", price: 19.99})\n\n# Update a product\n{:ok, product} = Inventory.update_product(product, %{price: 29.99})\n\n# Delete a product\n{:ok, _deleted} = Inventory.delete_product(product)\n```","ref":"overview.html#context-generation-with-aurora-ctx"},{"type":"extras","title":"Function Customization - Overview","doc":"Aurora.Ctx respects existing function definitions. You can provide custom implementations for specific operations while letting Aurora.Ctx generate the rest. Functions can be defined anywhere in the module:\n\n```elixir\ndefmodule MyApp.Inventory do\n  use Aurora.Ctx\n  ctx_register_schema(MyApp.Inventory.Product)\n\n  def list_products do\n    # Custom implementation with specific business logic\n    MyApp.Inventory.Product\n    |> where([p], p.active == true)\n    |> order_by([p], [desc: p.inserted_at])\n    |> MyApp.Repo.all()\n  end\nend\n```","ref":"overview.html#function-customization"},{"type":"extras","title":"Changeset Customization - Overview","doc":"Aurora.Ctx provides several options for customizing which changeset functions are used:\n\n```elixir\ndefmodule MyApp.Inventory do\n  use Aurora.Ctx\n\n  # Default changeset for all operations\n  ctx_register_schema(Product, changeset: :custom_changeset)\n\n  # Separate changesets for create/update\n  ctx_register_schema(Product,\n    create_changeset: :creation_changeset,\n    update_changeset: :update_changeset\n  )\nend\n```\n\nThe priority for selecting changeset functions is:\n1. Operation-specific changeset (create_changeset/update_changeset)\n2. Default changeset specified with :changeset option\n3. Standard \"changeset/2\" function in the schema","ref":"overview.html#changeset-customization"},{"type":"extras","title":"Generated Functions - Overview","doc":"For each schema, the following functions are automatically generated:","ref":"overview.html#generated-functions"},{"type":"extras","title":"List Functions - Overview","doc":"- `list_*(opts \\\\ nil)` - List all records with optional filtering/sorting\n- `list_*_paginated(opts \\\\ %{})` - List records with pagination and options\n- `count_*(opts \\\\ nil)` - Count records with optional filtering","ref":"overview.html#list-functions"},{"type":"extras","title":"Pagination Functions - Overview","doc":"- `to_*_page(pagination, page_number)` - Navigates to the specified page number\n- `next_*_page(pagination)` - Advances to the next page of results\n- `previous_*_page(pagination)` - Goes back to the previous page of results","ref":"overview.html#pagination-functions"},{"type":"extras","title":"Create Functions - Overview","doc":"- `create_*(attrs \\\\ %{})` - Create a record with optional attributes\n- `create_*!(attrs \\\\ %{})` - Create a record with optional attributes (raises on error)","ref":"overview.html#create-functions"},{"type":"extras","title":"Get Functions - Overview","doc":"- `get_*(id, opts \\\\ [])` - Get a record by ID with optional preloads\n- `get_*!(id, opts \\\\ [])` - Get a record by ID with optional preloads (raises if not found)","ref":"overview.html#get-functions"},{"type":"extras","title":"Delete Functions - Overview","doc":"- `delete_*(entity)` - Delete the given record\n- `delete_*!(entity)` - Delete the given record (raises on error)","ref":"overview.html#delete-functions"},{"type":"extras","title":"Change Functions - Overview","doc":"- `change_*(entity_or_changeset, attrs \\\\ %{})` - Create a changeset from an entity or existing changeset with optional attributes","ref":"overview.html#change-functions"},{"type":"extras","title":"Update Functions - Overview","doc":"- `update_*(entity_or_changeset)` - Update a record from an entity or changeset\n- `update_*(entity_or_changeset, attrs)` - Update a record with attributes, accepts entity or changeset\n- `update_*!(entity_or_changeset)` - Update a record from an entity or changeset (raises on error)\n- `update_*!(entity_or_changeset, attrs)` - Update a record with attributes, accepts entity or changeset (raises on error)","ref":"overview.html#update-functions"},{"type":"extras","title":"New Functions - Overview","doc":"- `new_*()` - Initialize a new struct\n- `new_*(attrs)` - Initialize a new struct with attributes\n- `new_*(attrs, opts)` - Initialize a new struct with attributes and options\n\nCheck the [Functions](functions.html) guide for detailed documentation of each function.","ref":"overview.html#new-functions"},{"type":"extras","title":"Direct Core Usage - Overview","doc":"Sometimes you may want to use the core operations directly without generating context functions. This approach is particularly useful in scenarios such as:\n\n- When working with dynamic schemas determined at runtime\n- Building generic admin interfaces or APIs that handle multiple schemas\n- Creating reusable modules that work with any schema\n- Testing and prototyping before defining your final context structure\n\nHere's how to use the core module directly:\n\n```elixir\nalias Aurora.Ctx.Core\n\n# List all records\nproducts = Core.list(MyApp.Repo, MyApp.Inventory.Product)\n\n# List with filtering and sorting\nproducts = Core.list(MyApp.Repo, MyApp.Inventory.Product,\n  where: [status: :active],\n  order_by: [desc: :inserted_at],\n  preload: [:category]\n)\n\n# Paginated listing\npage = Core.list_paginated(MyApp.Repo, MyApp.Inventory.Product,\n  paginate: %{page: 1, per_page: 20}\n)\n\n# Create a record\n{:ok, product} = Core.create(MyApp.Repo, MyApp.Inventory.Product, %{\n  name: \"Widget\",\n  price: 19.99\n})\n\n# Update a record\n{:ok, product} = Core.update(MyApp.Repo, product, %{price: 29.99})\n\n# Delete a record\n{:ok, _deleted} = Core.delete(MyApp.Repo, product)\n```","ref":"overview.html#direct-core-usage"},{"type":"extras","title":"Context vs Direct Core Usage - Overview","doc":"While using the Core module directly provides flexibility, using proper contexts has several advantages:\n\n1. **Encapsulation**: Contexts hide implementation details and provide a cleaner API\n2. **Business Logic**: Contexts are the ideal place to add business rules and validations\n3. **Consistency**: Generated functions ensure consistent naming and behavior across your app\n4. **Documentation**: Function names in contexts are more descriptive of their business purpose\n5. **Maintainability**: Changes to database operations can be centralized in contexts\n\nConsider using proper contexts for most application code, and reserve direct Core usage for special cases where flexibility is more important than the benefits listed above.","ref":"overview.html#context-vs-direct-core-usage"},{"type":"extras","title":"Generated Functions Reference","doc":"# Generated Functions Reference\n\nThis guide details all the functions that Aurora.Ctx generates for your context modules. To illustrate the generated artifacts, we use, as an example, a schema module named 'Product' that has a table named 'products'.","ref":"functions.html"},{"type":"extras","title":"List Functions - Generated Functions Reference","doc":"Functions to fetch and filter collections of records from the database.\n\n```elixir\nlist_products()                # Returns [%Product{}]\nlist_products(opts)            # Returns [%Product{}] with filtering/sorting\nlist_products_paginated()      # Returns %Pagination{} with default options\nlist_products_paginated(opts)  # Returns %Pagination{} with custom options\ncount_products()               # Returns total count of records\ncount_products(opts)           # Returns filtered count of records\n```","ref":"functions.html#list-functions"},{"type":"extras","title":"Example - Generated Functions Reference","doc":"```elixir\n# Get active products sorted by price\nproducts = list_products(\n  where: [status: :active],\n  order_by: [asc: :price]\n)\n```\n\nThe `opts` parameter supports:\n- `:preload` - Associations to preload\n- `:where` - Filter conditions (equality, comparison, range)\n- `:or_where` - Alternative filter conditions combined with OR\n- `:order_by` - Sorting specification\n- `:paginate` - Pagination options (page, per_page)\n- `:limit` - Maximum number of records to return","ref":"functions.html#example"},{"type":"extras","title":"Configuration - Generated Functions Reference","doc":"Default pagination settings can be configured in your `config.exs`:\n\n```elixir\nconfig :aurora_ctx,\n  paginate: %{\n    page: 1,\n    per_page: 40  # default if not specified\n  }\n```","ref":"functions.html#configuration"},{"type":"extras","title":"Count Functions - Generated Functions Reference","doc":"The count functions support the same filtering options as list functions:\n\n```elixir\n# Get total count\ntotal = count_products()\n\n# Get filtered count\nactive_count = count_products(where: [status: :active])\n```","ref":"functions.html#count-functions"},{"type":"extras","title":"Pagination Functions - Generated Functions Reference","doc":"Functions to navigate through large datasets efficiently. See `Aurora.Ctx.Pagination` module for the pagination implementation details.\n\n```elixir\nto_products_page(pagination, page)     # Returns %Pagination{} for specific page\nnext_products_page(pagination)         # Returns %Pagination{} for next page\nprevious_products_page(pagination)     # Returns %Pagination{} for previous page\n```","ref":"functions.html#pagination-functions"},{"type":"extras","title":"Example - Generated Functions Reference","doc":"```elixir\n# Get paginated results\npage = list_products_paginated(paginate: %{page: 1, per_page: 20})\n\n# Navigate pages\nnext_page = next_products_page(page)\nprev_page = previous_products_page(page)\npage5 = to_products_page(page, 5)\n```","ref":"functions.html#example-1"},{"type":"extras","title":"Get Functions - Generated Functions Reference","doc":"Functions to retrieve individual records by their primary key.\n\n```elixir\nget_product(id)                # Returns %Product{} or nil\nget_product(id, opts)          # Returns %Product{} or nil with preloads\nget_product!(id)               # Returns %Product{} or raises Ecto.NoResultsError\nget_product!(id, opts)         # Returns %Product{} or raises, with given options\n```","ref":"functions.html#get-functions"},{"type":"extras","title":"Example - Generated Functions Reference","doc":"```elixir\n# Get product with associated category\nproduct = get_product!(1, preload: [:category])\n```","ref":"functions.html#example-2"},{"type":"extras","title":"Create Functions - Generated Functions Reference","doc":"Functions to insert new records into the database.\n\n```elixir\ncreate_product()               # Returns {:ok, %Product{}} with defaults\ncreate_product(attrs)          # Returns {:ok, %Product{}} or {:error, changeset}\ncreate_product!()              # Returns %Product{} or raises errors\ncreate_product!(attrs)         # Returns %Product{} or raises errors\n```\n\nYou can customize which changeset function is used for creation by providing the `:create_changeset` option when registering the schema.","ref":"functions.html#create-functions"},{"type":"extras","title":"Example - Generated Functions Reference","doc":"```elixir\n{:ok, product} = create_product(%{\n  name: \"Widget Pro\",\n  price: Decimal.new(\"29.99\")\n})\n\n# Using a custom create changeset\ndefmodule MyApp.Inventory do\n  use Aurora.Ctx\n  ctx_register_schema(Product, create_changeset: :create_changeset)\nend\n```","ref":"functions.html#example-3"},{"type":"extras","title":"Update Functions - Generated Functions Reference","doc":"Functions to modify existing records in the database.\n\n```elixir\nupdate_product(entity)           # Returns {:ok, %Product{}} with no changes\nupdate_product(entity, attrs)    # Returns {:ok, %Product{}} or {:error, changeset}\nupdate_product(changeset)        # Returns {:ok, %Product{}} or {:error, changeset}\nupdate_product(changeset, attrs) # Returns {:ok, %Product{}} or {:error, changeset}\n```\n\nThe function accepts either:\n- An entity and optional attributes to apply changes\n- A pre-built changeset to validate and persist\n- A pre-built changeset and additional attributes to merge\n\n> **Note**: When using a pre-built changeset, it will be re-validated using the schema's defined `:update_changeset` function (or `:changeset` if not specified).\n\nYou can use a custom update changeset function by providing the `:update_changeset` option when registering the schema.","ref":"functions.html#update-functions"},{"type":"extras","title":"Example - Generated Functions Reference","doc":"```elixir\n# Using entity and attributes\n{:ok, updated} = update_product(product, %{price: Decimal.new(\"39.99\")})\n\n# Using a pre-built changeset\nchangeset = change_product(product, %{price: Decimal.new(\"39.99\")})\n{:ok, updated} = update_product(changeset)\n\n# Using a changeset with additional attributes\n{:ok, updated} = \n  product\n  |> change_product()\n  |> update_product(%{price: Decimal.new(\"39.99\")})\n\n# Using a custom update changeset\ndefmodule MyApp.Inventory do\n  use Aurora.Ctx\n  ctx_register_schema(Product, update_changeset: :custom_update_changeset)\nend\n```","ref":"functions.html#example-4"},{"type":"extras","title":"Delete Functions - Generated Functions Reference","doc":"Functions to remove records from the database.\n\n```elixir\ndelete_product(entity)         # Returns {:ok, %Product{}} or {:error, changeset}\ndelete_product!(entity)        # Returns %Product{} or raises\n```","ref":"functions.html#delete-functions"},{"type":"extras","title":"Example - Generated Functions Reference","doc":"```elixir\n{:ok, deleted} = delete_product(product)\n```","ref":"functions.html#example-5"},{"type":"extras","title":"Change Functions - Generated Functions Reference","doc":"Functions to prepare and validate changes before persisting them.\n\n```elixir\nchange_product(entity)               # Returns %Ecto.Changeset{} with no changes\nchange_product(entity, attrs)        # Returns %Ecto.Changeset{} with changes\nchange_product(changeset)            # Returns %Ecto.Changeset{} with no changes\nchange_product(changeset, attrs)     # Returns %Ecto.Changeset{} with changes\n```\n\nThe function accepts either:\n- An entity and optional attributes to apply changes\n- An existing changeset and optional attributes to merge changes\n\n> **Note**: When using a pre-built changeset, it will be re-validated using the schema's defined `:changeset` function (or the default changeset function if not specified).\n\nYou can customize which changeset function is used by providing the `:changeset` option when registering the schema.","ref":"functions.html#change-functions"},{"type":"extras","title":"Example - Generated Functions Reference","doc":"```elixir\n# Using entity\nchangeset = change_product(product, %{price: Decimal.new(\"49.99\")})\n\n# Using existing changeset\nupdated_changeset = \n  product\n  |> change_product(%{name: \"Widget Pro\"})\n  |> change_product(%{price: Decimal.new(\"49.99\")})\n\n# Using custom changeset function\ndefmodule MyApp.Inventory do\n  use Aurora.Ctx\n  ctx_register_schema(Product, changeset: :custom_changeset)\nend\n```","ref":"functions.html#example-6"},{"type":"extras","title":"New Functions - Generated Functions Reference","doc":"Functions to create new struct instances in memory.\n\n```elixir\nnew_product()                  # Returns %Product{} struct\nnew_product(attrs)             # Returns %Product{} with attributes\nnew_product(attrs, opts)       # Returns %Product{} with attributes and options applied\n```\n\nThe `opts` parameter supports:\n- `:preload` - Associations to preload when creating the struct","ref":"functions.html#new-functions"},{"type":"extras","title":"Example - Generated Functions Reference","doc":"```elixir\n# Basic usage\nproduct = new_product(%{name: \"Widget Pro\", price: Decimal.new(\"29.99\")})\n\n# With preloaded associations\nproduct = new_product(\n  %{name: \"Widget Pro\", price: Decimal.new(\"29.99\")},\n  preload: [:category, :variants]\n)\n```\n\n> **Note**: The `:changeset` option when registering a schema sets the default changeset function for all operations. This default is used for create and update operations unless explicitly overridden by `:create_changeset` or `:update_changeset` respectively.","ref":"functions.html#example-7"},{"type":"extras","title":"Query Options - Generated Functions Reference","doc":"The following options are available for list, get, and count functions:","ref":"functions.html#query-options"},{"type":"extras","title":"Where Conditions - Generated Functions Reference","doc":"```elixir\n# Basic equality\nwhere: [status: :active]\nwhere: {:status, :active}\n\n# Comparisons\nwhere: {:price, :greater_than, 100}      # or :gt\nwhere: {:price, :greater_equal_than, 100} # or :ge\nwhere: {:price, :less_than, 200}         # or :lt\nwhere: {:price, :less_equal_than, 200}   # or :le\nwhere: {:price, :equal_to, 150}          # or :eq\n\n# Ranges\nwhere: {:price, :between, 100, 200}\n\n# Multiple conditions (AND)\nwhere: [\n  status: :active,\n  {:price, :greater_than, 100}\n]\n\n# OR conditions\nwhere: [status: :active],\nor_where: [status: :pending]\n```","ref":"functions.html#where-conditions"},{"type":"extras","title":"Preloading - Generated Functions Reference","doc":"```elixir\n# Basic preloads\npreload: [:category]\n\n# Nested preloads\npreload: [\n  category: [:parent_category],\n  reviews: [:user, comments: [:user]]\n]\n\n# With query customization\npreload: [\n  reviews: from(r in Review, where: r.rating > 3)\n]\n```","ref":"functions.html#preloading"},{"type":"extras","title":"Sorting - Generated Functions Reference","doc":"```elixir\n# Basic sorting\norder_by: :inserted_at                 # asc\norder_by: {:desc, :price}             # desc\n\n# Null handling\norder_by: {:asc_nulls_last, :ended_at}\norder_by: {:desc_nulls_first, :priority}\n\n# Multiple fields\norder_by: [\n  {:desc, :priority},\n  {:asc, :name}\n]\n```","ref":"functions.html#sorting"},{"type":"extras","title":"Query Exclusions - Generated Functions Reference","doc":"You can exclude specific query clauses when needed:\n\n```elixir\n# Exclude specific clauses\nproducts = list_products(\n  where: [status: :active],\n  exclude: :where  # Removes where clause\n)\n\n# Exclude multiple clauses\nproducts = list_products(\n  where: [status: :active],\n  order_by: [desc: :inserted_at],\n  exclude: [:where, :order_by]\n)\n```","ref":"functions.html#query-exclusions"},{"type":"extras","title":"Pagination - Generated Functions Reference","doc":"```elixir\n# Basic pagination\npaginate: %{page: 1, per_page: 20}\n```\n\nFor implementation examples, check out the [Examples](examples.html) guide.","ref":"functions.html#pagination"},{"type":"extras","title":"Examples","doc":"# Examples\n\nExamples of using Aurora.Ctx in common scenarios.","ref":"examples.html"},{"type":"extras","title":"Basic CRUD Operations - Examples","doc":"Basic context setup and standard database operations.\n\n```elixir\ndefmodule MyApp.Inventory do\n  use Aurora.Ctx,\n    schema_module: MyApp.Inventory.Product,\n    repo_module: MyApp.Repo\nend\n\n# Usage examples\nalias MyApp.Inventory\n\n# Create a product\n{:ok, product} = Inventory.create_product(%{\n  name: \"Premium Widget\",\n  price: Decimal.new(\"29.99\")\n})\n\n# List products with sorting\nproducts = Inventory.list_products(order_by: [desc: :inserted_at])\n\n# Get product with preloaded associations\n{:ok, product} = Inventory.get_product(1, preload: [:category, :supplier])\n\n# Update a product\n{:ok, updated} = Inventory.update_product(product, %{price: Decimal.new(\"39.99\")})\n\n# Delete a product\n{:ok, _deleted} = Inventory.delete_product(product)\n```","ref":"examples.html#basic-crud-operations"},{"type":"extras","title":"Query Usage - Examples","doc":"More complex database operations using query options.","ref":"examples.html#query-usage"},{"type":"extras","title":"Custom Query Options - Examples","doc":"Combining filtering, preloading, and sorting in queries.\n\n```elixir\n# List active products with preloaded categories\nproducts = Inventory.list_products(\n  where: [status: \"active\"],\n  preload: [:category],\n  order_by: [asc: :name],\n  limit: 10\n)\n\n# Get a product with specific preloads\nproduct = Inventory.get_product!(1, \n  preload: [\n    category: [:parent_category],\n    reviews: [:user]\n  ]\n)\n```","ref":"examples.html#custom-query-options"},{"type":"extras","title":"Working with Changesets - Examples","doc":"Data validation and form handling with changesets.\n\n```elixir\n# Create a changeset for a new product\nchangeset = Inventory.change_product(%Product{})\n\n# Create a changeset with attributes\nchangeset = Inventory.change_product(%Product{}, %{\n  name: \"New Widget\",\n  price: \"19.99\"\n})\n\n# Working with forms in Phoenix\ndef new(conn, _params) do\n  changeset = Inventory.change_product(%Product{})\n  render(conn, \"new.html\", changeset: changeset)\nend\n\ndef create(conn, %{\"product\" => product_params}) do\n  case Inventory.create_product(product_params) do\n    {:ok, product} -> \n      redirect(to: Routes.product_path(conn, :show, product))\n    {:error, changeset} ->\n      render(conn, \"new.html\", changeset: changeset)\n  end\nend\n```","ref":"examples.html#working-with-changesets"},{"type":"extras","title":"Query Options - Examples","doc":"Available options for customizing database queries.","ref":"examples.html#query-options"},{"type":"extras","title":"Filtering - Examples","doc":"Filter query results using different conditions.\n\n```elixir\n# Multiple conditions\nproducts = Inventory.list_products(\n  where: [\n    status: \"active\",\n    category_id: 1\n  ]\n)\n\n# Comparison operators\nproducts = Inventory.list_products(\n  where: [\n    {:price, :gt, Decimal.new(\"100.00\")},\n    {:stock_level, :le, 5}\n  ]\n)\n\n# Range queries\nproducts = Inventory.list_products(\n  where: {:price, :between, Decimal.new(\"10.00\"), Decimal.new(\"50.00\")}\n)\n\n# OR conditions\nproducts = Inventory.list_products(\n  where: [status: \"active\"],\n  or_where: [status: \"pending_review\"]\n)\n```","ref":"examples.html#filtering"},{"type":"extras","title":"Sorting - Examples","doc":"Order query results with various sorting strategies.\n\n```elixir\n# Multiple sort fields\nproducts = Inventory.list_products(\n  order_by: [\n    asc: :category_id,\n    desc: :price,\n    asc: :name\n  ]\n)\n\n# Null handling\nproducts = Inventory.list_products(\n  order_by: [\n    asc_nulls_last: :discontinued_at,\n    desc_nulls_first: :updated_at\n  ]\n)\n```","ref":"examples.html#sorting"},{"type":"extras","title":"Controlled Pagination - Examples","doc":"Implement and navigate through paginated results. Results are wrapped in an `Aurora.Ctx.Pagination` struct that provides additional metadata and navigation helpers.\n\n```elixir\n# Basic pagination with options\npage1 = Inventory.list_products_paginated(\n  paginate: %{page: 1, per_page: 20},\n  preload: [:category],\n  order_by: [desc: :inserted_at]\n)\n\n# Access entries and metadata\nEnum.each(page1.entries, fn product ->\n  IO.puts \"#{product.name}: $#{product.price}\"\nend)\nIO.puts \"Total entries: #{page1.entries_count}\"\nIO.puts \"Total pages: #{page1.pages_count}\"\n\n# Navigate through pages\nnext_page = Inventory.next_products_page(page1)\nprev_page = Inventory.previous_products_page(next_page)\npage5 = Inventory.to_products_page(page1, 5)\n```\n\n#### Phoenix Example\n\nController:\n```elixir\ndef index(conn, params) do\n  pagination = Inventory.list_products_paginated(\n    paginate: %{page: params[\"page\"] || 1, per_page: 20},\n    order_by: [desc: :inserted_at]\n  )\n  render(conn, :index, pagination: pagination)\nend\n\ndef page(conn, %{\"page\" => page}) do\n  pagination = conn.assigns.pagination\n  new_page = Inventory.to_products_page(pagination, page)\n  render(conn, :index, pagination: new_page)\nend\n\ndef next_page(conn, _params) do\n  pagination = conn.assigns.pagination\n  next_page = Inventory.next_products_page(pagination)\n  render(conn, :index, pagination: next_page)\nend\n\ndef previous_page(conn, _params) do\n  pagination = conn.assigns.pagination\n  prev_page = Inventory.previous_products_page(pagination)\n  render(conn, :index, pagination: prev_page)\nend\n```\n\nTemplate:\n```heex\n \n  <%= for product <- @pagination.entries do %>\n     \n       <%= product.name %> \n       $<%= product.price %> \n     \n  <% end %>\n \n\n \n  <%= if @pagination.page > 1 do %>\n    <%= link \"Previous\", to: ~p\"/products/previous\" %>\n  <% end %>\n  \n   \n    Page <%= @pagination.page %> of <%= @pagination.pages_count %>\n    (<%= @pagination.entries_count %> total items)\n   \n  \n  <%= if @pagination.page < @pagination.pages_count do %>\n    <%= link \"Next\", to: ~p\"/products/next\" %>\n  <% end %>\n  \n   \n    <%= for page <- 1..@pagination.pages_count do %>\n      <%= link \"#{page}\", to: ~p\"/products/page/#{page}\", \n          class: if(@pagination.page == page, do: \"active\") %>\n    <% end %>\n   \n \n```","ref":"examples.html#controlled-pagination"},{"type":"extras","title":"Preloading Associates - Examples","doc":"Load associated data with simple and nested preloads.\n\n```elixir\n# Nested preloads\nproduct = Inventory.get_product!(1,\n  preload: [\n    category: [:parent_category, :subcategories],\n    reviews: [:user, comments: [:user]],\n    supplier: [:address, :contacts]\n  ]\n)\n\n# Filtered preloads with custom queries\nproducts = Inventory.list_products(\n  preload: [\n    reviews: from(r in Review, where: r.rating > 3)\n  ]\n)\n```","ref":"examples.html#preloading-associates"},{"type":"extras","title":"Working with Transactions - Examples","doc":"Combine multiple database operations in transactions.\n\n```elixir\nalias Ecto.Multi\nalias MyApp.{Inventory, Orders}\n\n# Complex multi-operation transaction\ndef create_order_with_inventory_update(attrs) do\n  Multi.new()\n  |> Multi.run(:product, fn repo, _ ->\n    Inventory.get_product(attrs.product_id)\n  end)\n  |> Multi.run(:check_stock, fn _repo, %{product: product} ->\n    if product.stock >= attrs.quantity do\n      {:ok, product}\n    else\n      {:error, :insufficient_stock}\n    end\n  end)\n  |> Multi.run(:order, fn repo, %{product: product} ->\n    Orders.create_order(attrs)\n  end)\n  |> Multi.run(:update_inventory, fn repo, %{product: product, order: order} ->\n    Inventory.update_product(product, %{\n      stock: product.stock - order.quantity\n    })\n  end)\n  |> MyApp.Repo.transaction()\nend\n\n# Usage\ncase create_order_with_inventory_update(%{product_id: 1, quantity: 5}) do\n  {:ok, %{order: order, product: product}} ->\n    {:ok, order}\n  {:error, failed_operation, failed_value, changes_so_far} ->\n    {:error, :order_creation_failed}\nend\n```","ref":"examples.html#working-with-transactions"},{"type":"extras","title":"Custom Changesets - Examples","doc":"Configure and use custom changeset functions.\n\n```elixir\n# Using custom changeset functions\ndefmodule MyApp.Inventory do\n  use Aurora.Ctx,\n    schema_module: MyApp.Inventory.Product,\n    repo_module: MyApp.Repo,\n    update_changeset: :custom_changeset,\n    create_changeset: :create_changeset\n\n  # Your additional context functions...\nend\n\n# Schema with multiple changeset functions\ndefmodule MyApp.Inventory.Product do\n  use Ecto.Schema\n  import Ecto.Changeset\n\n  schema \"products\" do\n    field :name, :string\n    field :price, :decimal\n    field :status, :string\n    timestamps()\n  end\n\n  def create_changeset(product, attrs) do\n    product\n    |> cast(attrs, [:name, :price])\n    |> validate_required([:name, :price])\n    |> validate_number(:price, greater_than: 0)\n  end\n\n  def custom_changeset(product, attrs) do\n    product\n    |> cast(attrs, [:name, :price, :status])\n    |> validate_required([:status])\n    |> validate_inclusion(:status, [\"active\", \"inactive\"])\n  end\nend\n\n# Usage of custom changesets\n# Create with custom create_changeset\n{:ok, product} = Inventory.create_product(%{\n  name: \"New Product\",\n  price: Decimal.new(\"29.99\")\n})\n\n# Update with custom_changeset\n{:ok, updated} = Inventory.update_product(product, %{\n  status: \"active\"\n})\n\n# Get changeset for form\nchangeset = Inventory.change_product(product, %{status: \"inactive\"})\n```\n\nFor more examples and use cases, check the test files in the [GitHub repository](https://github.com/wadvanced/aurora_ctx).","ref":"examples.html#custom-changesets"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}